# coding: utf-8

"""
    FlashArray REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 2.11
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re

# python 2 and python 3 compatibility library
import six
from typing import List, Optional

from .. import models

class SoftwareApi(object):

    def __init__(self, api_client):
        self.api_client = api_client

    def api211_software_bundle_get_with_http_info(
        self,
        filter=None,  # type: str
        ids=None,  # type: List[str]
        limit=None,  # type: int
        offset=None,  # type: int
        sort=None,  # type: List[str]
        total_item_count=None,  # type: bool
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareBundleGetResponse
        """List software-bundle

        Displays a list of software bundles.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_bundle_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str filter: Narrows down the results to only the response objects that satisfy the filter criteria.
        :param list[str] ids: Performs the operation on the unique resource IDs specified. Enter multiple resource IDs in comma-separated format. The `ids` or `names` parameter is required, but they cannot be set together.
        :param int limit: Limits the size of the response to the specified number of objects on each page. To return the total number of resources, set `limit=0`. The total number of resources is returned as a `total_item_count` value. If the page size requested is larger than the system maximum limit, the server returns the maximum limit, disregarding the requested page size.
        :param int offset: The starting position based on the results of the query in relation to the full set of response objects returned.
        :param list[str] sort: Returns the response objects in the order specified. Set `sort` to the name in the response by which to sort. Sorting can be performed on any of the names in the response, and the objects can be sorted in ascending or descending order. By default, the response objects are sorted in ascending order. To sort in descending order, append the minus sign (`-`) to the name. A single request can be sorted on multiple objects. For example, you can sort all volumes from largest to smallest volume size, and then sort volumes of the same size in ascending order by volume name. To sort on multiple names, list the names as comma-separated values.
        :param bool total_item_count: If set to `true`, the `total_item_count` matching the specified query parameters is calculated and returned in the response. If set to `false`, the `total_item_count` is `null` in the response. This may speed up queries where the `total_item_count` is large. If not specified, defaults to `false`.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareBundleGetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if ids is not None:
            if not isinstance(ids, list):
                ids = [ids]
        if sort is not None:
            if not isinstance(sort, list):
                sort = [sort]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        if 'limit' in params and params['limit'] < 1:
            raise ValueError("Invalid value for parameter `limit` when calling `api211_software_bundle_get`, must be a value greater than or equal to `1`")
        if 'offset' in params and params['offset'] < 0:
            raise ValueError("Invalid value for parameter `offset` when calling `api211_software_bundle_get`, must be a value greater than or equal to `0`")
        collection_formats = {}
        path_params = {}

        query_params = []
        if 'filter' in params:
            query_params.append(('filter', params['filter']))
        if 'ids' in params:
            query_params.append(('ids', params['ids']))
            collection_formats['ids'] = 'csv'
        if 'limit' in params:
            query_params.append(('limit', params['limit']))
        if 'offset' in params:
            query_params.append(('offset', params['offset']))
        if 'sort' in params:
            query_params.append(('sort', params['sort']))
            collection_formats['sort'] = 'csv'
        if 'total_item_count' in params:
            query_params.append(('total_item_count', params['total_item_count']))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-bundle', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareBundleGetResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_bundle_post_with_http_info(
        self,
        source=None,  # type: models.SoftwareBundlePost
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareBundleResponse
        """Create software-bundle

        Creates and initiates a software bundle download.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_bundle_post_with_http_info(source, async_req=True)
        >>> result = thread.get()

        :param SoftwareBundlePost source: (required)
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareBundleResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]
        # verify the required parameter 'source' is set
        if source is None:
            raise TypeError("Missing the required parameter `source` when calling `api211_software_bundle_post`")

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'source' in params:
            body_params = params['source']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-bundle', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareBundleResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_check_delete_with_http_info(
        self,
        software_versions=None,  # type: List[str]
        software_names=None,  # type: List[str]
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Delete a software check task

        Deletes a software check task specified by software name and version. The check task must be queued. If the check task is running or passed/failed then the task cannot be deleted.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_check_delete_with_http_info(async_req=True)
        >>> result = thread.get()

        :param list[str] software_versions: A comma-separated list of target software versions.
        :param list[str] software_names: A comma-separated list of software names.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if software_versions is not None:
            if not isinstance(software_versions, list):
                software_versions = [software_versions]
        if software_names is not None:
            if not isinstance(software_names, list):
                software_names = [software_names]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        collection_formats = {}
        path_params = {}

        query_params = []
        if 'software_versions' in params:
            query_params.append(('software_versions', params['software_versions']))
            collection_formats['software_versions'] = 'csv'
        if 'software_names' in params:
            query_params.append(('software_names', params['software_names']))
            collection_formats['software_names'] = 'csv'

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-check', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_check_get_with_http_info(
        self,
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        filter=None,  # type: str
        ids=None,  # type: List[str]
        limit=None,  # type: int
        names=None,  # type: List[str]
        offset=None,  # type: int
        software_versions=None,  # type: List[str]
        software_names=None,  # type: List[str]
        sort=None,  # type: List[str]
        total_item_count=None,  # type: bool
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareChecksGetResponse
        """List software check tasks

        Displays a list of software check tasks.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_check_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param str filter: Narrows down the results to only the response objects that satisfy the filter criteria.
        :param list[str] ids: Performs the operation on the unique resource IDs specified. Enter multiple resource IDs in comma-separated format. The `ids` or `names` parameter is required, but they cannot be set together.
        :param int limit: Limits the size of the response to the specified number of objects on each page. To return the total number of resources, set `limit=0`. The total number of resources is returned as a `total_item_count` value. If the page size requested is larger than the system maximum limit, the server returns the maximum limit, disregarding the requested page size.
        :param list[str] names: Performs the operation on the unique name specified. Enter multiple names in comma-separated format. For example, `name01,name02`.
        :param int offset: The starting position based on the results of the query in relation to the full set of response objects returned.
        :param list[str] software_versions: A comma-separated list of target software versions.
        :param list[str] software_names: A comma-separated list of software names.
        :param list[str] sort: Returns the response objects in the order specified. Set `sort` to the name in the response by which to sort. Sorting can be performed on any of the names in the response, and the objects can be sorted in ascending or descending order. By default, the response objects are sorted in ascending order. To sort in descending order, append the minus sign (`-`) to the name. A single request can be sorted on multiple objects. For example, you can sort all volumes from largest to smallest volume size, and then sort volumes of the same size in ascending order by volume name. To sort on multiple names, list the names as comma-separated values.
        :param bool total_item_count: If set to `true`, the `total_item_count` matching the specified query parameters is calculated and returned in the response. If set to `false`, the `total_item_count` is `null` in the response. This may speed up queries where the `total_item_count` is large. If not specified, defaults to `false`.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareChecksGetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if ids is not None:
            if not isinstance(ids, list):
                ids = [ids]
        if names is not None:
            if not isinstance(names, list):
                names = [names]
        if software_versions is not None:
            if not isinstance(software_versions, list):
                software_versions = [software_versions]
        if software_names is not None:
            if not isinstance(software_names, list):
                software_names = [software_names]
        if sort is not None:
            if not isinstance(sort, list):
                sort = [sort]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        if 'limit' in params and params['limit'] < 1:
            raise ValueError("Invalid value for parameter `limit` when calling `api211_software_check_get`, must be a value greater than or equal to `1`")
        if 'offset' in params and params['offset'] < 0:
            raise ValueError("Invalid value for parameter `offset` when calling `api211_software_check_get`, must be a value greater than or equal to `0`")
        collection_formats = {}
        path_params = {}

        query_params = []
        if 'filter' in params:
            query_params.append(('filter', params['filter']))
        if 'ids' in params:
            query_params.append(('ids', params['ids']))
            collection_formats['ids'] = 'csv'
        if 'limit' in params:
            query_params.append(('limit', params['limit']))
        if 'names' in params:
            query_params.append(('names', params['names']))
            collection_formats['names'] = 'csv'
        if 'offset' in params:
            query_params.append(('offset', params['offset']))
        if 'software_versions' in params:
            query_params.append(('software_versions', params['software_versions']))
            collection_formats['software_versions'] = 'csv'
        if 'software_names' in params:
            query_params.append(('software_names', params['software_names']))
            collection_formats['software_names'] = 'csv'
        if 'sort' in params:
            query_params.append(('sort', params['sort']))
            collection_formats['sort'] = 'csv'
        if 'total_item_count' in params:
            query_params.append(('total_item_count', params['total_item_count']))

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-check', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareChecksGetResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_check_post_with_http_info(
        self,
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        software_versions=None,  # type: List[str]
        software_names=None,  # type: List[str]
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareChecksResponse
        """Create a software check task

        Creates a software check task. If there are no previously queued or running software check tasks, the new task immediately starts. Otherwise, it is queued and starts once all previously queued or running software check tasks finish. To create a task, use a software name and version.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_check_post_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param list[str] software_versions: A comma-separated list of target software versions.
        :param list[str] software_names: A comma-separated list of software names.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareChecksResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if software_versions is not None:
            if not isinstance(software_versions, list):
                software_versions = [software_versions]
        if software_names is not None:
            if not isinstance(software_names, list):
                software_names = [software_names]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        collection_formats = {}
        path_params = {}

        query_params = []
        if 'software_versions' in params:
            query_params.append(('software_versions', params['software_versions']))
            collection_formats['software_versions'] = 'csv'
        if 'software_names' in params:
            query_params.append(('software_names', params['software_names']))
            collection_formats['software_names'] = 'csv'

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-check', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareChecksResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_delete_with_http_info(
        self,
        names=None,  # type: List[str]
        software_versions=None,  # type: List[str]
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> None
        """Delete a software package

        Deletes a software package specified by software name and version. A software package being used for an installation can be deleted if the installation is `downloading`, `downloaded`, `aborted`, or `finished`. If the software package is currently `downloading`, the download will be cancelled.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_delete_with_http_info(async_req=True)
        >>> result = thread.get()

        :param list[str] names: Performs the operation on the unique name specified. Enter multiple names in comma-separated format. For example, `name01,name02`.
        :param list[str] software_versions: A comma-separated list of target software versions.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if names is not None:
            if not isinstance(names, list):
                names = [names]
        if software_versions is not None:
            if not isinstance(software_versions, list):
                software_versions = [software_versions]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        collection_formats = {}
        path_params = {}

        query_params = []
        if 'names' in params:
            query_params.append(('names', params['names']))
            collection_formats['names'] = 'csv'
        if 'software_versions' in params:
            query_params.append(('software_versions', params['software_versions']))
            collection_formats['software_versions'] = 'csv'

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_get_with_http_info(
        self,
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        filter=None,  # type: str
        ids=None,  # type: List[str]
        limit=None,  # type: int
        names=None,  # type: List[str]
        offset=None,  # type: int
        sort=None,  # type: List[str]
        total_item_count=None,  # type: bool
        versions=None,  # type: List[str]
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareGetResponse
        """List software packages

        Displays a list of available software packages.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param str filter: Narrows down the results to only the response objects that satisfy the filter criteria.
        :param list[str] ids: Performs the operation on the unique resource IDs specified. Enter multiple resource IDs in comma-separated format. The `ids` or `names` parameter is required, but they cannot be set together.
        :param int limit: Limits the size of the response to the specified number of objects on each page. To return the total number of resources, set `limit=0`. The total number of resources is returned as a `total_item_count` value. If the page size requested is larger than the system maximum limit, the server returns the maximum limit, disregarding the requested page size.
        :param list[str] names: Performs the operation on the unique name specified. Enter multiple names in comma-separated format. For example, `name01,name02`.
        :param int offset: The starting position based on the results of the query in relation to the full set of response objects returned.
        :param list[str] sort: Returns the response objects in the order specified. Set `sort` to the name in the response by which to sort. Sorting can be performed on any of the names in the response, and the objects can be sorted in ascending or descending order. By default, the response objects are sorted in ascending order. To sort in descending order, append the minus sign (`-`) to the name. A single request can be sorted on multiple objects. For example, you can sort all volumes from largest to smallest volume size, and then sort volumes of the same size in ascending order by volume name. To sort on multiple names, list the names as comma-separated values.
        :param bool total_item_count: If set to `true`, the `total_item_count` matching the specified query parameters is calculated and returned in the response. If set to `false`, the `total_item_count` is `null` in the response. This may speed up queries where the `total_item_count` is large. If not specified, defaults to `false`.
        :param list[str] versions: A comma-separated list of versions.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareGetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if ids is not None:
            if not isinstance(ids, list):
                ids = [ids]
        if names is not None:
            if not isinstance(names, list):
                names = [names]
        if sort is not None:
            if not isinstance(sort, list):
                sort = [sort]
        if versions is not None:
            if not isinstance(versions, list):
                versions = [versions]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        if 'limit' in params and params['limit'] < 1:
            raise ValueError("Invalid value for parameter `limit` when calling `api211_software_get`, must be a value greater than or equal to `1`")
        if 'offset' in params and params['offset'] < 0:
            raise ValueError("Invalid value for parameter `offset` when calling `api211_software_get`, must be a value greater than or equal to `0`")
        collection_formats = {}
        path_params = {}

        query_params = []
        if 'filter' in params:
            query_params.append(('filter', params['filter']))
        if 'ids' in params:
            query_params.append(('ids', params['ids']))
            collection_formats['ids'] = 'csv'
        if 'limit' in params:
            query_params.append(('limit', params['limit']))
        if 'names' in params:
            query_params.append(('names', params['names']))
            collection_formats['names'] = 'csv'
        if 'offset' in params:
            query_params.append(('offset', params['offset']))
        if 'sort' in params:
            query_params.append(('sort', params['sort']))
            collection_formats['sort'] = 'csv'
        if 'total_item_count' in params:
            query_params.append(('total_item_count', params['total_item_count']))
        if 'versions' in params:
            query_params.append(('versions', params['versions']))
            collection_formats['versions'] = 'csv'

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareGetResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_installation_steps_get_with_http_info(
        self,
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        continuation_token=None,  # type: str
        filter=None,  # type: str
        ids=None,  # type: List[str]
        limit=None,  # type: int
        names=None,  # type: List[str]
        offset=None,  # type: int
        software_installation_ids=None,  # type: List[str]
        sort=None,  # type: List[str]
        total_item_count=None,  # type: bool
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareInstallationStepsGetResponse
        """List software upgrade steps

        Displays a list of currently running and completed software upgrade steps.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_installation_steps_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param str continuation_token: A token used to retrieve the next page of data with some consistency guaranteed. The token is a Base64 encoded value. Set `continuation_token` to the system-generated token taken from the `x-next-token` header field of the response. A query has reached its last page when the response does not include a token. Pagination requires the `limit` and `continuation_token` query parameters.
        :param str filter: Narrows down the results to only the response objects that satisfy the filter criteria.
        :param list[str] ids: Performs the operation on the unique resource IDs specified. Enter multiple resource IDs in comma-separated format. The `ids` or `names` parameter is required, but they cannot be set together.
        :param int limit: Limits the size of the response to the specified number of objects on each page. To return the total number of resources, set `limit=0`. The total number of resources is returned as a `total_item_count` value. If the page size requested is larger than the system maximum limit, the server returns the maximum limit, disregarding the requested page size.
        :param list[str] names: Performs the operation on the unique name specified. Enter multiple names in comma-separated format. For example, `name01,name02`.
        :param int offset: The starting position based on the results of the query in relation to the full set of response objects returned.
        :param list[str] software_installation_ids: A comma-separated list of software installation IDs.
        :param list[str] sort: Returns the response objects in the order specified. Set `sort` to the name in the response by which to sort. Sorting can be performed on any of the names in the response, and the objects can be sorted in ascending or descending order. By default, the response objects are sorted in ascending order. To sort in descending order, append the minus sign (`-`) to the name. A single request can be sorted on multiple objects. For example, you can sort all volumes from largest to smallest volume size, and then sort volumes of the same size in ascending order by volume name. To sort on multiple names, list the names as comma-separated values.
        :param bool total_item_count: If set to `true`, the `total_item_count` matching the specified query parameters is calculated and returned in the response. If set to `false`, the `total_item_count` is `null` in the response. This may speed up queries where the `total_item_count` is large. If not specified, defaults to `false`.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareInstallationStepsGetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if ids is not None:
            if not isinstance(ids, list):
                ids = [ids]
        if names is not None:
            if not isinstance(names, list):
                names = [names]
        if software_installation_ids is not None:
            if not isinstance(software_installation_ids, list):
                software_installation_ids = [software_installation_ids]
        if sort is not None:
            if not isinstance(sort, list):
                sort = [sort]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        if 'limit' in params and params['limit'] < 1:
            raise ValueError("Invalid value for parameter `limit` when calling `api211_software_installation_steps_get`, must be a value greater than or equal to `1`")
        if 'offset' in params and params['offset'] < 0:
            raise ValueError("Invalid value for parameter `offset` when calling `api211_software_installation_steps_get`, must be a value greater than or equal to `0`")
        collection_formats = {}
        path_params = {}

        query_params = []
        if 'continuation_token' in params:
            query_params.append(('continuation_token', params['continuation_token']))
        if 'filter' in params:
            query_params.append(('filter', params['filter']))
        if 'ids' in params:
            query_params.append(('ids', params['ids']))
            collection_formats['ids'] = 'csv'
        if 'limit' in params:
            query_params.append(('limit', params['limit']))
        if 'names' in params:
            query_params.append(('names', params['names']))
            collection_formats['names'] = 'csv'
        if 'offset' in params:
            query_params.append(('offset', params['offset']))
        if 'software_installation_ids' in params:
            query_params.append(('software_installation_ids', params['software_installation_ids']))
            collection_formats['software_installation_ids'] = 'csv'
        if 'sort' in params:
            query_params.append(('sort', params['sort']))
            collection_formats['sort'] = 'csv'
        if 'total_item_count' in params:
            query_params.append(('total_item_count', params['total_item_count']))

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-installation-steps', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareInstallationStepsGetResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_installations_get_with_http_info(
        self,
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        continuation_token=None,  # type: str
        filter=None,  # type: str
        ids=None,  # type: List[str]
        limit=None,  # type: int
        names=None,  # type: List[str]
        offset=None,  # type: int
        software_ids=None,  # type: List[str]
        software_names=None,  # type: List[str]
        sort=None,  # type: List[str]
        total_item_count=None,  # type: bool
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareInstallationsGetResponse
        """List software upgrades

        Displays a list of software upgrades, including currently running and past upgrades.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_installations_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param str continuation_token: A token used to retrieve the next page of data with some consistency guaranteed. The token is a Base64 encoded value. Set `continuation_token` to the system-generated token taken from the `x-next-token` header field of the response. A query has reached its last page when the response does not include a token. Pagination requires the `limit` and `continuation_token` query parameters.
        :param str filter: Narrows down the results to only the response objects that satisfy the filter criteria.
        :param list[str] ids: Performs the operation on the unique resource IDs specified. Enter multiple resource IDs in comma-separated format. The `ids` or `names` parameter is required, but they cannot be set together.
        :param int limit: Limits the size of the response to the specified number of objects on each page. To return the total number of resources, set `limit=0`. The total number of resources is returned as a `total_item_count` value. If the page size requested is larger than the system maximum limit, the server returns the maximum limit, disregarding the requested page size.
        :param list[str] names: Performs the operation on the unique name specified. Enter multiple names in comma-separated format. For example, `name01,name02`.
        :param int offset: The starting position based on the results of the query in relation to the full set of response objects returned.
        :param list[str] software_ids: A comma-separated list of software IDs.
        :param list[str] software_names: A comma-separated list of software names.
        :param list[str] sort: Returns the response objects in the order specified. Set `sort` to the name in the response by which to sort. Sorting can be performed on any of the names in the response, and the objects can be sorted in ascending or descending order. By default, the response objects are sorted in ascending order. To sort in descending order, append the minus sign (`-`) to the name. A single request can be sorted on multiple objects. For example, you can sort all volumes from largest to smallest volume size, and then sort volumes of the same size in ascending order by volume name. To sort on multiple names, list the names as comma-separated values.
        :param bool total_item_count: If set to `true`, the `total_item_count` matching the specified query parameters is calculated and returned in the response. If set to `false`, the `total_item_count` is `null` in the response. This may speed up queries where the `total_item_count` is large. If not specified, defaults to `false`.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareInstallationsGetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if ids is not None:
            if not isinstance(ids, list):
                ids = [ids]
        if names is not None:
            if not isinstance(names, list):
                names = [names]
        if software_ids is not None:
            if not isinstance(software_ids, list):
                software_ids = [software_ids]
        if software_names is not None:
            if not isinstance(software_names, list):
                software_names = [software_names]
        if sort is not None:
            if not isinstance(sort, list):
                sort = [sort]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        if 'limit' in params and params['limit'] < 1:
            raise ValueError("Invalid value for parameter `limit` when calling `api211_software_installations_get`, must be a value greater than or equal to `1`")
        if 'offset' in params and params['offset'] < 0:
            raise ValueError("Invalid value for parameter `offset` when calling `api211_software_installations_get`, must be a value greater than or equal to `0`")
        collection_formats = {}
        path_params = {}

        query_params = []
        if 'continuation_token' in params:
            query_params.append(('continuation_token', params['continuation_token']))
        if 'filter' in params:
            query_params.append(('filter', params['filter']))
        if 'ids' in params:
            query_params.append(('ids', params['ids']))
            collection_formats['ids'] = 'csv'
        if 'limit' in params:
            query_params.append(('limit', params['limit']))
        if 'names' in params:
            query_params.append(('names', params['names']))
            collection_formats['names'] = 'csv'
        if 'offset' in params:
            query_params.append(('offset', params['offset']))
        if 'software_ids' in params:
            query_params.append(('software_ids', params['software_ids']))
            collection_formats['software_ids'] = 'csv'
        if 'software_names' in params:
            query_params.append(('software_names', params['software_names']))
            collection_formats['software_names'] = 'csv'
        if 'sort' in params:
            query_params.append(('sort', params['sort']))
            collection_formats['sort'] = 'csv'
        if 'total_item_count' in params:
            query_params.append(('total_item_count', params['total_item_count']))

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-installations', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareInstallationsGetResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_installations_patch_with_http_info(
        self,
        command=None,  # type: str
        current_step_id=None,  # type: str
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        software_installations=None,  # type: models.SoftwareInstallationPatch
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareInstallationsResponse
        """Modify software upgrade

        Modifies a software upgrade by continuing, retrying, or aborting it. All `override_checks` are updated before the command being issued if `add_override_checks` is present. The `override_checks` parameter is valid when `command` is `continue` or `retry`.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_installations_patch_with_http_info(command, current_step_id, async_req=True)
        >>> result = thread.get()

        :param str command: A user command that interacts with the upgrade. Commands may only be issued when the upgrade is paused. Valid values are `continue`, `retry`, and `abort`. The `continue` command continues a `paused` upgrade. The `retry` command retries the previous step. The `abort` command aborts the upgrade. (required)
        :param str current_step_id: The current step `id` of the installation. (required)
        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param SoftwareInstallationPatch software_installations:
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareInstallationsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]
        # verify the required parameter 'command' is set
        if command is None:
            raise TypeError("Missing the required parameter `command` when calling `api211_software_installations_patch`")
        # verify the required parameter 'current_step_id' is set
        if current_step_id is None:
            raise TypeError("Missing the required parameter `current_step_id` when calling `api211_software_installations_patch`")

        collection_formats = {}
        path_params = {}

        query_params = []
        if 'command' in params:
            query_params.append(('command', params['command']))
        if 'current_step_id' in params:
            query_params.append(('current_step_id', params['current_step_id']))

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        if 'software_installations' in params:
            body_params = params['software_installations']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-installations', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareInstallationsResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_installations_post_with_http_info(
        self,
        software_ids=None,  # type: List[str]
        software_installations=None,  # type: models.SoftwareInstallationPost
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareInstallationsResponse
        """Create a software upgrade

        Creates and initiates a software upgrade.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_installations_post_with_http_info(software_ids, software_installations, async_req=True)
        >>> result = thread.get()

        :param list[str] software_ids: A comma-separated list of software IDs. (required)
        :param SoftwareInstallationPost software_installations: (required)
        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareInstallationsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if software_ids is not None:
            if not isinstance(software_ids, list):
                software_ids = [software_ids]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]
        # verify the required parameter 'software_ids' is set
        if software_ids is None:
            raise TypeError("Missing the required parameter `software_ids` when calling `api211_software_installations_post`")
        # verify the required parameter 'software_installations' is set
        if software_installations is None:
            raise TypeError("Missing the required parameter `software_installations` when calling `api211_software_installations_post`")

        collection_formats = {}
        path_params = {}

        query_params = []
        if 'software_ids' in params:
            query_params.append(('software_ids', params['software_ids']))
            collection_formats['software_ids'] = 'csv'

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        if 'software_installations' in params:
            body_params = params['software_installations']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-installations', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareInstallationsResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_post_with_http_info(
        self,
        software=None,  # type: models.SoftwarePost
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareResponse
        """Create a software package

        Creates a software package download.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_post_with_http_info(software, async_req=True)
        >>> result = thread.get()

        :param SoftwarePost software: (required)
        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]
        # verify the required parameter 'software' is set
        if software is None:
            raise TypeError("Missing the required parameter `software` when calling `api211_software_post`")

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        if 'software' in params:
            body_params = params['software']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api211_software_versions_get_with_http_info(
        self,
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        filter=None,  # type: str
        limit=None,  # type: int
        offset=None,  # type: int
        sort=None,  # type: List[str]
        total_item_count=None,  # type: bool
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.SoftwareVersionsGetResponse
        """List software versions

        Displays a list of software versions.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api211_software_versions_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param str filter: Narrows down the results to only the response objects that satisfy the filter criteria.
        :param int limit: Limits the size of the response to the specified number of objects on each page. To return the total number of resources, set `limit=0`. The total number of resources is returned as a `total_item_count` value. If the page size requested is larger than the system maximum limit, the server returns the maximum limit, disregarding the requested page size.
        :param int offset: The starting position based on the results of the query in relation to the full set of response objects returned.
        :param list[str] sort: Returns the response objects in the order specified. Set `sort` to the name in the response by which to sort. Sorting can be performed on any of the names in the response, and the objects can be sorted in ascending or descending order. By default, the response objects are sorted in ascending order. To sort in descending order, append the minus sign (`-`) to the name. A single request can be sorted on multiple objects. For example, you can sort all volumes from largest to smallest volume size, and then sort volumes of the same size in ascending order by volume name. To sort on multiple names, list the names as comma-separated values.
        :param bool total_item_count: If set to `true`, the `total_item_count` matching the specified query parameters is calculated and returned in the response. If set to `false`, the `total_item_count` is `null` in the response. This may speed up queries where the `total_item_count` is large. If not specified, defaults to `false`.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: SoftwareVersionsGetResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if sort is not None:
            if not isinstance(sort, list):
                sort = [sort]
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]

        if 'limit' in params and params['limit'] < 1:
            raise ValueError("Invalid value for parameter `limit` when calling `api211_software_versions_get`, must be a value greater than or equal to `1`")
        if 'offset' in params and params['offset'] < 0:
            raise ValueError("Invalid value for parameter `offset` when calling `api211_software_versions_get`, must be a value greater than or equal to `0`")
        collection_formats = {}
        path_params = {}

        query_params = []
        if 'filter' in params:
            query_params.append(('filter', params['filter']))
        if 'limit' in params:
            query_params.append(('limit', params['limit']))
        if 'offset' in params:
            query_params.append(('offset', params['offset']))
        if 'sort' in params:
            query_params.append(('sort', params['sort']))
            collection_formats['sort'] = 'csv'
        if 'total_item_count' in params:
            query_params.append(('total_item_count', params['total_item_count']))

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.11/software-versions', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SoftwareVersionsGetResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )



# coding: utf-8

"""
    FlashBlade REST API

    A lightweight client for FlashBlade REST API 2.5, developed by Pure Storage, Inc. (http://www.purestorage.com/).

    OpenAPI spec version: 2.5
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flashblade.FB_2_5 import models

class ApiClient(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'id': 'str',
        'access_token_ttl_in_ms': 'int',
        'enabled': 'bool',
        'issuer': 'str',
        'key_id': 'str',
        'max_role': 'FixedReference',
        'public_key': 'str'
    }

    attribute_map = {
        'name': 'name',
        'id': 'id',
        'access_token_ttl_in_ms': 'access_token_ttl_in_ms',
        'enabled': 'enabled',
        'issuer': 'issuer',
        'key_id': 'key_id',
        'max_role': 'max_role',
        'public_key': 'public_key'
    }

    required_args = {
    }

    def __init__(
        self,
        name=None,  # type: str
        id=None,  # type: str
        access_token_ttl_in_ms=None,  # type: int
        enabled=None,  # type: bool
        issuer=None,  # type: str
        key_id=None,  # type: str
        max_role=None,  # type: models.FixedReference
        public_key=None,  # type: str
    ):
        """
        Keyword args:
            name (str): Name of the object (e.g., a file system or snapshot).
            id (str): A non-modifiable, globally unique ID chosen by the system.
            access_token_ttl_in_ms (int): The TTL (Time To Live) duration for which the exchanged access token is valid. Measured in milliseconds.
            enabled (bool): If `true`, the API client is permitted to exchange ID Tokens for access tokens. API clients are disabled by default.
            issuer (str): The name of the identity provider that will be issuing ID Tokens for this API client. This string represents the JWT `iss` (issuer) claim in ID Tokens issued for this API client.
            key_id (str): The unique identifier for the associated public key of this API client. This string must match the JWT `kid` (key ID) claim in ID Tokens issued for this API client.
            max_role (FixedReference): The maximum role allowed for ID Tokens issued by this API client. The bearer of an access token will be authorized to perform actions within the intersection of this `max_role` and the role of the array user specified as the JWT `sub` (subject) claim. The `max_role` field is a reference to a `role`. Valid `role`s are `readonly`, `ops_admin`, `array_admin`, and `storage_admin`. Users with the `readonly` (Read Only) role can perform operations that convey the state of the array. Read Only users cannot alter the state of the array. Users with the `ops_admin` (Ops Admin) role can perform the same operations as Read Only users plus enable and disable remote assistance sessions. Ops Admin users cannot alter the state of the array. Users with the `storage_admin` (Storage Admin) role can perform the same operations as Read Only users plus storage related operations, such as administering file systems, snapshots, and buckets. Storage Admin users cannot perform operations that deal with global and system configurations. Users with the `array_admin` (Array Admin) role can perform the same operations as Storage Admin users plus array-wide changes dealing with global and system configurations. In other words, Array Admin users can perform all operations.
            public_key (str): The API client's PEM formatted (Base64 encoded) RSA public key.
        """
        if name is not None:
            self.name = name
        if id is not None:
            self.id = id
        if access_token_ttl_in_ms is not None:
            self.access_token_ttl_in_ms = access_token_ttl_in_ms
        if enabled is not None:
            self.enabled = enabled
        if issuer is not None:
            self.issuer = issuer
        if key_id is not None:
            self.key_id = key_id
        if max_role is not None:
            self.max_role = max_role
        if public_key is not None:
            self.public_key = public_key

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `ApiClient`".format(key))
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            return None
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(ApiClient, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ApiClient):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashArray REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 2.21
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flasharray.FA_2_21 import models

class Directory(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'name': 'str',
        'created': 'int',
        'destroyed': 'bool',
        'directory_name': 'str',
        'file_system': 'FixedReference',
        'path': 'str',
        'space': 'Space',
        'time_remaining': 'int',
        'limited_by': 'LimitedBy'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'created': 'created',
        'destroyed': 'destroyed',
        'directory_name': 'directory_name',
        'file_system': 'file_system',
        'path': 'path',
        'space': 'space',
        'time_remaining': 'time_remaining',
        'limited_by': 'limited_by'
    }

    required_args = {
    }

    def __init__(
        self,
        id=None,  # type: str
        name=None,  # type: str
        created=None,  # type: int
        destroyed=None,  # type: bool
        directory_name=None,  # type: str
        file_system=None,  # type: models.FixedReference
        path=None,  # type: str
        space=None,  # type: models.Space
        time_remaining=None,  # type: int
        limited_by=None,  # type: models.LimitedBy
    ):
        """
        Keyword args:
            id (str): A globally unique, system-generated ID. The ID cannot be modified and cannot refer to another resource.
            name (str): A user-specified name. The name must be locally unique and can be changed.
            created (int): The managed directory creation time, measured in milliseconds since the UNIX epoch.
            destroyed (bool): Returns a value of `true` if the managed directory has been destroyed and is pending eradication. The `time_remaining` value displays the amount of time left until the destroyed managed directory is permanently eradicated. Once the `time_remaining` period has elapsed, the managed directory is permanently eradicated and can no longer be recovered.
            directory_name (str): The managed directory name without the file system name prefix. A full managed directory name is constructed in the form of `FILE_SYSTEM:DIR` where `FILE_SYSTEM` is the file system name and `DIR` is the value of this field.
            file_system (FixedReference): The file system that this managed directory is in.
            path (str): Absolute path of the managed directory in the file system.
            space (Space): Displays size and space consumption information.
            time_remaining (int): The amount of time left, measured in milliseconds until the destroyed managed directory is permanently eradicated.
            limited_by (LimitedBy): The quota policy that is limiting usage on this managed directory. This policy defines the total amount of space provisioned to this managed directory and its descendants. The returned value contains two parts&#58; the name of the policy and the managed directory to which the policy is attached.
        """
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if created is not None:
            self.created = created
        if destroyed is not None:
            self.destroyed = destroyed
        if directory_name is not None:
            self.directory_name = directory_name
        if file_system is not None:
            self.file_system = file_system
        if path is not None:
            self.path = path
        if space is not None:
            self.space = space
        if time_remaining is not None:
            self.time_remaining = time_remaining
        if limited_by is not None:
            self.limited_by = limited_by

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `Directory`".format(key))
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            raise AttributeError
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(Directory, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Directory):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashArray REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 2.10
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re

# python 2 and python 3 compatibility library
import six
from typing import List, Optional

from .. import models

class VirtualMachinesApi(object):

    def __init__(self, api_client):
        self.api_client = api_client

    def api210_virtual_machines_patch_with_http_info(
        self,
        virtual_machine=None,  # type: models.VirtualMachinePost
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.VirtualMachineResponse
        """Modify a virtual machine

        Modifies a virtual machine, recovering it from the destroyed state. If recovering the virtual machine causes a conflict with an existing virtual machine, the operation fails.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api210_virtual_machines_patch_with_http_info(virtual_machine, async_req=True)
        >>> result = thread.get()

        :param VirtualMachinePost virtual_machine: (required)
        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: VirtualMachineResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]
        # verify the required parameter 'virtual_machine' is set
        if virtual_machine is None:
            raise TypeError("Missing the required parameter `virtual_machine` when calling `api210_virtual_machines_patch`")

        collection_formats = {}
        path_params = {}

        query_params = []

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        if 'virtual_machine' in params:
            body_params = params['virtual_machine']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.10/virtual-machines', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='VirtualMachineResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )

    def api210_virtual_machines_post_with_http_info(
        self,
        virtual_machine=None,  # type: models.VirtualMachinePost
        authorization=None,  # type: str
        x_request_id=None,  # type: str
        overwrite=None,  # type: bool
        async_req=False,  # type: bool
        _return_http_data_only=False,  # type: bool
        _preload_content=True,  # type: bool
        _request_timeout=None,  # type: Optional[int]
    ):
        # type: (...) -> models.VirtualMachineResponse
        """Create a virtual machine

        Creates one or more virtual machines from a protection group snapshot. If `overwrite` is specified, an existing virtual machine has its volumes overwritten by the snapshot. Otherwise, a new virtual machine is created from the snapshot. If creating the new virtual machine causes a conflict with an existing virtual machine, the operation fails.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.api210_virtual_machines_post_with_http_info(virtual_machine, async_req=True)
        >>> result = thread.get()

        :param VirtualMachinePost virtual_machine: (required)
        :param str authorization: Access token (in JWT format) required to use any API endpoint (except `/oauth2`, `/login`, and `/logout`)
        :param str x_request_id: Supplied by client during request or generated by server.
        :param bool overwrite: If set to `true`, overwrites an existing volume during a volume copy operation. If set to `false` or not set at all and the target name is an existing volume, the volume copy operation fails. Required if the `source: id` or `source: name` body parameter is set and the source overwrites an existing volume during the volume copy operation.
        :param bool async_req: Request runs in separate thread and method returns multiprocessing.pool.ApplyResult.
        :param bool _return_http_data_only: Returns only data field.
        :param bool _preload_content: Response is converted into objects.
        :param int _request_timeout: Total request timeout in seconds.
                 It can also be a tuple of (connection time, read time) timeouts.
        :return: VirtualMachineResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        params = {k: v for k, v in six.iteritems(locals()) if v is not None}

        # Convert the filter into a string
        if params.get('filter'):
            params['filter'] = str(params['filter'])
        if params.get('sort'):
            params['sort'] = [str(_x) for _x in params['sort']]
        # verify the required parameter 'virtual_machine' is set
        if virtual_machine is None:
            raise TypeError("Missing the required parameter `virtual_machine` when calling `api210_virtual_machines_post`")

        collection_formats = {}
        path_params = {}

        query_params = []
        if 'overwrite' in params:
            query_params.append(('overwrite', params['overwrite']))

        header_params = {}
        if 'authorization' in params:
            header_params['Authorization'] = params['authorization']
        if 'x_request_id' in params:
            header_params['X-Request-ID'] = params['x_request_id']

        form_params = []
        local_var_files = {}

        body_params = None
        if 'virtual_machine' in params:
            body_params = params['virtual_machine']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/api/2.10/virtual-machines', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='VirtualMachineResponse',
            auth_settings=auth_settings,
            async_req=async_req,
            _return_http_data_only=_return_http_data_only,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
            collection_formats=collection_formats,
        )



# coding: utf-8

"""
    FlashBlade REST API

    A lightweight client for FlashBlade REST API 2.7, developed by Pure Storage, Inc. (http://www.purestorage.com/).

    OpenAPI spec version: 2.7
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flashblade.FB_2_7 import models

class Array(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'id': 'str',
        'as_of': 'int',
        'banner': 'str',
        'idle_timeout': 'int',
        'ntp_servers': 'list[str]',
        'os': 'str',
        'revision': 'str',
        'time_zone': 'str',
        'version': 'str',
        'smb_mode': 'str',
        'eradication_config': 'ArrayEradicationConfig',
        'product_type': 'str',
        'security_update': 'str'
    }

    attribute_map = {
        'name': 'name',
        'id': 'id',
        'as_of': '_as_of',
        'banner': 'banner',
        'idle_timeout': 'idle_timeout',
        'ntp_servers': 'ntp_servers',
        'os': 'os',
        'revision': 'revision',
        'time_zone': 'time_zone',
        'version': 'version',
        'smb_mode': 'smb_mode',
        'eradication_config': 'eradication_config',
        'product_type': 'product_type',
        'security_update': 'security_update'
    }

    required_args = {
    }

    def __init__(
        self,
        name=None,  # type: str
        id=None,  # type: str
        as_of=None,  # type: int
        banner=None,  # type: str
        idle_timeout=None,  # type: int
        ntp_servers=None,  # type: List[str]
        os=None,  # type: str
        revision=None,  # type: str
        time_zone=None,  # type: str
        version=None,  # type: str
        smb_mode=None,  # type: str
        eradication_config=None,  # type: models.ArrayEradicationConfig
        product_type=None,  # type: str
        security_update=None,  # type: str
    ):
        """
        Keyword args:
            name (str): A name chosen by the user. Can be changed. Must be locally unique.
            id (str): A non-modifiable, globally unique ID chosen by the system.
            as_of (int): Array time in milliseconds since UNIX epoch.
            banner (str): A string to be shown when logging in to the array.
            idle_timeout (int): Idle timeout in milliseconds. Valid values are in the range of 300000 and 10800000. Setting this value to 0 disables timeouts.
            ntp_servers (list[str])
            os (str): Valid values are `Purity//FA` and `Purity//FB`.
            revision (str)
            time_zone (str): The time zone to use for the array. In particular this affects the CLI formatting and the default snapshot policy time zone.
            version (str)
            smb_mode (str): The current SMB mode of the array. This controls the type of authentication that is used by the array for SMB. Possible values include `ad-auto`, `ad-rfc2307`, `guest`, and `native`. Modifying this value requires the assistance of Pure Storage Support.
            eradication_config (ArrayEradicationConfig)
            product_type (str): For `FlashBlade//S` arrays, the value is determined by the blades in the system. The value will be `FlashBlade` for all older arrays. Valid values are `FlashBlade`, `FlashBlade//S`, `FlashBlade//S200`, and `FlashBladeS500`.
            security_update (str): The name of the installed security update that currently applies to the system. This field will be `null` if either no security update has been installed, or if the most recently installed security update is no longer needed by the current FlashBlade software due to the current software fully incorporating the update.
        """
        if name is not None:
            self.name = name
        if id is not None:
            self.id = id
        if as_of is not None:
            self.as_of = as_of
        if banner is not None:
            self.banner = banner
        if idle_timeout is not None:
            self.idle_timeout = idle_timeout
        if ntp_servers is not None:
            self.ntp_servers = ntp_servers
        if os is not None:
            self.os = os
        if revision is not None:
            self.revision = revision
        if time_zone is not None:
            self.time_zone = time_zone
        if version is not None:
            self.version = version
        if smb_mode is not None:
            self.smb_mode = smb_mode
        if eradication_config is not None:
            self.eradication_config = eradication_config
        if product_type is not None:
            self.product_type = product_type
        if security_update is not None:
            self.security_update = security_update

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `Array`".format(key))
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            return None
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(Array, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Array):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashBlade REST API

    A lightweight client for FlashBlade REST API 2.7, developed by Pure Storage, Inc. (http://www.purestorage.com/).

    OpenAPI spec version: 2.7
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flashblade.FB_2_7 import models

class FileSystemPatch(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'id': 'str',
        'created': 'int',
        'default_group_quota': 'int',
        'default_user_quota': 'int',
        'destroyed': 'bool',
        'fast_remove_directory_enabled': 'bool',
        'hard_limit_enabled': 'bool',
        'http': 'Http',
        'multi_protocol': 'MultiProtocol',
        'nfs': 'NfsPatch',
        'promotion_status': 'str',
        'provisioned': 'int',
        'requested_promotion_state': 'str',
        'smb': 'Smb',
        'snapshot_directory_enabled': 'bool',
        'source': 'LocationReference',
        'space': 'Space',
        'time_remaining': 'int',
        'writable': 'bool'
    }

    attribute_map = {
        'name': 'name',
        'id': 'id',
        'created': 'created',
        'default_group_quota': 'default_group_quota',
        'default_user_quota': 'default_user_quota',
        'destroyed': 'destroyed',
        'fast_remove_directory_enabled': 'fast_remove_directory_enabled',
        'hard_limit_enabled': 'hard_limit_enabled',
        'http': 'http',
        'multi_protocol': 'multi_protocol',
        'nfs': 'nfs',
        'promotion_status': 'promotion_status',
        'provisioned': 'provisioned',
        'requested_promotion_state': 'requested_promotion_state',
        'smb': 'smb',
        'snapshot_directory_enabled': 'snapshot_directory_enabled',
        'source': 'source',
        'space': 'space',
        'time_remaining': 'time_remaining',
        'writable': 'writable'
    }

    required_args = {
    }

    def __init__(
        self,
        name=None,  # type: str
        id=None,  # type: str
        created=None,  # type: int
        default_group_quota=None,  # type: int
        default_user_quota=None,  # type: int
        destroyed=None,  # type: bool
        fast_remove_directory_enabled=None,  # type: bool
        hard_limit_enabled=None,  # type: bool
        http=None,  # type: models.Http
        multi_protocol=None,  # type: models.MultiProtocol
        nfs=None,  # type: models.NfsPatch
        promotion_status=None,  # type: str
        provisioned=None,  # type: int
        requested_promotion_state=None,  # type: str
        smb=None,  # type: models.Smb
        snapshot_directory_enabled=None,  # type: bool
        source=None,  # type: models.LocationReference
        space=None,  # type: models.Space
        time_remaining=None,  # type: int
        writable=None,  # type: bool
    ):
        """
        Keyword args:
            name (str): Name of the object (e.g., a file system or snapshot).
            id (str): A non-modifiable, globally unique ID chosen by the system.
            created (int): Creation timestamp of the file system.
            default_group_quota (int): The default space quota for a group writing to this file system.
            default_user_quota (int): The default space quota for a user writing to this file system.
            destroyed (bool): Returns a value of `true` if the file system has been destroyed and is pending eradication. The file system cannot be modified while it is in the destroyed state. The `time_remaining` value displays the amount of time left until the destroyed file system is permanently eradicated. Once eradication has begun, the file system can no longer be recovered. Before the `time_remaining` period has elapsed, the destroyed file system can be recovered through the PATCH method by setting `destroyed=false`.
            fast_remove_directory_enabled (bool): If set to `true`, the file system, when mounted, will contain a directory that can be used for fast removal of other directories. Directories can be moved into the fast remove directory in order to have them deleted, and their space freed, more quickly than a normal removal operation.
            hard_limit_enabled (bool): If set to `true`, the file system's size, as defined by `provisioned`, is used as a hard limit quota.
            http (Http): HTTP configuration.
            multi_protocol (MultiProtocol): Multi-protocol configuration.
            nfs (NfsPatch): NFS configuration.
            promotion_status (str): Possible values are `promoted` and `demoted`. The current status of the file system with respect to replication. Changes via `requested_promotion_state`. The default for new file systems is `promoted`.
            provisioned (int): The provisioned size of the file system, displayed in bytes. If set to an empty string (`\"\"`), the file system is unlimited in size.
            requested_promotion_state (str): Possible values are `promoted` and `demoted`. The `demoted` state is used for replication targets and is only allowed to be set if the file system is in a replica-link relationship. The additional query param `discard-non-snapshotted-data` must be set to `true` when demoting a file system. The default for new file systems is `promoted`.
            smb (Smb): SMB configuration.
            snapshot_directory_enabled (bool): If set to `true`, a hidden .snapshot directory will be present in each directory of the file system when it is mounted. The .snapshot directory allows clients read access to the contents of the snapshots that have been taken of a directory. If set to `false`, the .snapshot directory will not be present in any directories within a mounted file system.
            source (LocationReference): A reference to the source file system.
            space (Space): The space usage of the file system.
            time_remaining (int): Time in milliseconds before the file system is eradicated. `null` if not destroyed.
            writable (bool): Whether the file system is writable or not. If `false`, this overrides any protocol or file permission settings and prevents changes. If `true`, then the protocol and file permission settings are evaluated. If not specified, defaults to `true`. Modifiable.
        """
        if name is not None:
            self.name = name
        if id is not None:
            self.id = id
        if created is not None:
            self.created = created
        if default_group_quota is not None:
            self.default_group_quota = default_group_quota
        if default_user_quota is not None:
            self.default_user_quota = default_user_quota
        if destroyed is not None:
            self.destroyed = destroyed
        if fast_remove_directory_enabled is not None:
            self.fast_remove_directory_enabled = fast_remove_directory_enabled
        if hard_limit_enabled is not None:
            self.hard_limit_enabled = hard_limit_enabled
        if http is not None:
            self.http = http
        if multi_protocol is not None:
            self.multi_protocol = multi_protocol
        if nfs is not None:
            self.nfs = nfs
        if promotion_status is not None:
            self.promotion_status = promotion_status
        if provisioned is not None:
            self.provisioned = provisioned
        if requested_promotion_state is not None:
            self.requested_promotion_state = requested_promotion_state
        if smb is not None:
            self.smb = smb
        if snapshot_directory_enabled is not None:
            self.snapshot_directory_enabled = snapshot_directory_enabled
        if source is not None:
            self.source = source
        if space is not None:
            self.space = space
        if time_remaining is not None:
            self.time_remaining = time_remaining
        if writable is not None:
            self.writable = writable

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `FileSystemPatch`".format(key))
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            return None
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(FileSystemPatch, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FileSystemPatch):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashArray REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 2.11
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flasharray.FA_2_11 import models

class NetworkInterfacePerformanceEth(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'other_errors_per_sec': 'int',
        'received_bytes_per_sec': 'int',
        'received_crc_errors_per_sec': 'int',
        'received_frame_errors_per_sec': 'int',
        'received_packets_per_sec': 'int',
        'total_errors_per_sec': 'int',
        'transmitted_bytes_per_sec': 'int',
        'transmitted_carrier_errors_per_sec': 'int',
        'transmitted_dropped_errors_per_sec': 'int',
        'transmitted_packets_per_sec': 'int'
    }

    attribute_map = {
        'other_errors_per_sec': 'other_errors_per_sec',
        'received_bytes_per_sec': 'received_bytes_per_sec',
        'received_crc_errors_per_sec': 'received_crc_errors_per_sec',
        'received_frame_errors_per_sec': 'received_frame_errors_per_sec',
        'received_packets_per_sec': 'received_packets_per_sec',
        'total_errors_per_sec': 'total_errors_per_sec',
        'transmitted_bytes_per_sec': 'transmitted_bytes_per_sec',
        'transmitted_carrier_errors_per_sec': 'transmitted_carrier_errors_per_sec',
        'transmitted_dropped_errors_per_sec': 'transmitted_dropped_errors_per_sec',
        'transmitted_packets_per_sec': 'transmitted_packets_per_sec'
    }

    required_args = {
    }

    def __init__(
        self,
        other_errors_per_sec=None,  # type: int
        received_bytes_per_sec=None,  # type: int
        received_crc_errors_per_sec=None,  # type: int
        received_frame_errors_per_sec=None,  # type: int
        received_packets_per_sec=None,  # type: int
        total_errors_per_sec=None,  # type: int
        transmitted_bytes_per_sec=None,  # type: int
        transmitted_carrier_errors_per_sec=None,  # type: int
        transmitted_dropped_errors_per_sec=None,  # type: int
        transmitted_packets_per_sec=None,  # type: int
    ):
        """
        Keyword args:
            other_errors_per_sec (int): The sum of unspecified reception and transmission errors per second.
            received_bytes_per_sec (int): Bytes received per second.
            received_crc_errors_per_sec (int): Reception CRC errors per second.
            received_frame_errors_per_sec (int): Received packet frame errors per second.
            received_packets_per_sec (int): Packets received per second.
            total_errors_per_sec (int): The sum of all reception and transmission errors per second.
            transmitted_bytes_per_sec (int): Bytes transmitted per second.
            transmitted_carrier_errors_per_sec (int): Transmission carrier errors per second.
            transmitted_dropped_errors_per_sec (int): Transmitted packets dropped per second.
            transmitted_packets_per_sec (int): Packets transmitted per second.
        """
        if other_errors_per_sec is not None:
            self.other_errors_per_sec = other_errors_per_sec
        if received_bytes_per_sec is not None:
            self.received_bytes_per_sec = received_bytes_per_sec
        if received_crc_errors_per_sec is not None:
            self.received_crc_errors_per_sec = received_crc_errors_per_sec
        if received_frame_errors_per_sec is not None:
            self.received_frame_errors_per_sec = received_frame_errors_per_sec
        if received_packets_per_sec is not None:
            self.received_packets_per_sec = received_packets_per_sec
        if total_errors_per_sec is not None:
            self.total_errors_per_sec = total_errors_per_sec
        if transmitted_bytes_per_sec is not None:
            self.transmitted_bytes_per_sec = transmitted_bytes_per_sec
        if transmitted_carrier_errors_per_sec is not None:
            self.transmitted_carrier_errors_per_sec = transmitted_carrier_errors_per_sec
        if transmitted_dropped_errors_per_sec is not None:
            self.transmitted_dropped_errors_per_sec = transmitted_dropped_errors_per_sec
        if transmitted_packets_per_sec is not None:
            self.transmitted_packets_per_sec = transmitted_packets_per_sec

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `NetworkInterfacePerformanceEth`".format(key))
        if key == "other_errors_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `other_errors_per_sec`, must be a value greater than or equal to `0`")
        if key == "received_bytes_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `received_bytes_per_sec`, must be a value greater than or equal to `0`")
        if key == "received_crc_errors_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `received_crc_errors_per_sec`, must be a value greater than or equal to `0`")
        if key == "received_frame_errors_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `received_frame_errors_per_sec`, must be a value greater than or equal to `0`")
        if key == "received_packets_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `received_packets_per_sec`, must be a value greater than or equal to `0`")
        if key == "total_errors_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `total_errors_per_sec`, must be a value greater than or equal to `0`")
        if key == "transmitted_bytes_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `transmitted_bytes_per_sec`, must be a value greater than or equal to `0`")
        if key == "transmitted_carrier_errors_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `transmitted_carrier_errors_per_sec`, must be a value greater than or equal to `0`")
        if key == "transmitted_dropped_errors_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `transmitted_dropped_errors_per_sec`, must be a value greater than or equal to `0`")
        if key == "transmitted_packets_per_sec" and value is not None:
            if value < 0:
                raise ValueError("Invalid value for `transmitted_packets_per_sec`, must be a value greater than or equal to `0`")
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            raise AttributeError
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(NetworkInterfacePerformanceEth, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NetworkInterfacePerformanceEth):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashBlade REST API

    A lightweight client for FlashBlade REST API 2.2, developed by Pure Storage, Inc. (http://www.purestorage.com/).

    OpenAPI spec version: 2.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flashblade.FB_2_2 import models

class ArrayNfsSpecificPerformance(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'id': 'str',
        'accesses_per_sec': 'float',
        'aggregate_file_metadata_creates_per_sec': 'float',
        'aggregate_file_metadata_modifies_per_sec': 'float',
        'aggregate_file_metadata_reads_per_sec': 'float',
        'aggregate_other_per_sec': 'float',
        'aggregate_share_metadata_reads_per_sec': 'float',
        'aggregate_usec_per_file_metadata_create_op': 'float',
        'aggregate_usec_per_file_metadata_modify_op': 'float',
        'aggregate_usec_per_file_metadata_read_op': 'float',
        'aggregate_usec_per_other_op': 'float',
        'aggregate_usec_per_share_metadata_read_op': 'float',
        'creates_per_sec': 'float',
        'fsinfos_per_sec': 'float',
        'fsstats_per_sec': 'float',
        'getattrs_per_sec': 'float',
        'links_per_sec': 'float',
        'lookups_per_sec': 'float',
        'mkdirs_per_sec': 'float',
        'pathconfs_per_sec': 'float',
        'reads_per_sec': 'float',
        'readdirs_per_sec': 'float',
        'readdirpluses_per_sec': 'float',
        'readlinks_per_sec': 'float',
        'removes_per_sec': 'float',
        'renames_per_sec': 'float',
        'rmdirs_per_sec': 'float',
        'setattrs_per_sec': 'float',
        'symlinks_per_sec': 'float',
        'time': 'int',
        'writes_per_sec': 'float',
        'usec_per_access_op': 'float',
        'usec_per_create_op': 'float',
        'usec_per_fsinfo_op': 'float',
        'usec_per_fsstat_op': 'float',
        'usec_per_getattr_op': 'float',
        'usec_per_link_op': 'float',
        'usec_per_lookup_op': 'float',
        'usec_per_mkdir_op': 'float',
        'usec_per_pathconf_op': 'float',
        'usec_per_read_op': 'float',
        'usec_per_readdir_op': 'float',
        'usec_per_readdirplus_op': 'float',
        'usec_per_readlink_op': 'float',
        'usec_per_remove_op': 'float',
        'usec_per_rename_op': 'float',
        'usec_per_rmdir_op': 'float',
        'usec_per_setattr_op': 'float',
        'usec_per_symlink_op': 'float',
        'usec_per_write_op': 'float'
    }

    attribute_map = {
        'name': 'name',
        'id': 'id',
        'accesses_per_sec': 'accesses_per_sec',
        'aggregate_file_metadata_creates_per_sec': 'aggregate_file_metadata_creates_per_sec',
        'aggregate_file_metadata_modifies_per_sec': 'aggregate_file_metadata_modifies_per_sec',
        'aggregate_file_metadata_reads_per_sec': 'aggregate_file_metadata_reads_per_sec',
        'aggregate_other_per_sec': 'aggregate_other_per_sec',
        'aggregate_share_metadata_reads_per_sec': 'aggregate_share_metadata_reads_per_sec',
        'aggregate_usec_per_file_metadata_create_op': 'aggregate_usec_per_file_metadata_create_op',
        'aggregate_usec_per_file_metadata_modify_op': 'aggregate_usec_per_file_metadata_modify_op',
        'aggregate_usec_per_file_metadata_read_op': 'aggregate_usec_per_file_metadata_read_op',
        'aggregate_usec_per_other_op': 'aggregate_usec_per_other_op',
        'aggregate_usec_per_share_metadata_read_op': 'aggregate_usec_per_share_metadata_read_op',
        'creates_per_sec': 'creates_per_sec',
        'fsinfos_per_sec': 'fsinfos_per_sec',
        'fsstats_per_sec': 'fsstats_per_sec',
        'getattrs_per_sec': 'getattrs_per_sec',
        'links_per_sec': 'links_per_sec',
        'lookups_per_sec': 'lookups_per_sec',
        'mkdirs_per_sec': 'mkdirs_per_sec',
        'pathconfs_per_sec': 'pathconfs_per_sec',
        'reads_per_sec': 'reads_per_sec',
        'readdirs_per_sec': 'readdirs_per_sec',
        'readdirpluses_per_sec': 'readdirpluses_per_sec',
        'readlinks_per_sec': 'readlinks_per_sec',
        'removes_per_sec': 'removes_per_sec',
        'renames_per_sec': 'renames_per_sec',
        'rmdirs_per_sec': 'rmdirs_per_sec',
        'setattrs_per_sec': 'setattrs_per_sec',
        'symlinks_per_sec': 'symlinks_per_sec',
        'time': 'time',
        'writes_per_sec': 'writes_per_sec',
        'usec_per_access_op': 'usec_per_access_op',
        'usec_per_create_op': 'usec_per_create_op',
        'usec_per_fsinfo_op': 'usec_per_fsinfo_op',
        'usec_per_fsstat_op': 'usec_per_fsstat_op',
        'usec_per_getattr_op': 'usec_per_getattr_op',
        'usec_per_link_op': 'usec_per_link_op',
        'usec_per_lookup_op': 'usec_per_lookup_op',
        'usec_per_mkdir_op': 'usec_per_mkdir_op',
        'usec_per_pathconf_op': 'usec_per_pathconf_op',
        'usec_per_read_op': 'usec_per_read_op',
        'usec_per_readdir_op': 'usec_per_readdir_op',
        'usec_per_readdirplus_op': 'usec_per_readdirplus_op',
        'usec_per_readlink_op': 'usec_per_readlink_op',
        'usec_per_remove_op': 'usec_per_remove_op',
        'usec_per_rename_op': 'usec_per_rename_op',
        'usec_per_rmdir_op': 'usec_per_rmdir_op',
        'usec_per_setattr_op': 'usec_per_setattr_op',
        'usec_per_symlink_op': 'usec_per_symlink_op',
        'usec_per_write_op': 'usec_per_write_op'
    }

    required_args = {
    }

    def __init__(
        self,
        name=None,  # type: str
        id=None,  # type: str
        accesses_per_sec=None,  # type: float
        aggregate_file_metadata_creates_per_sec=None,  # type: float
        aggregate_file_metadata_modifies_per_sec=None,  # type: float
        aggregate_file_metadata_reads_per_sec=None,  # type: float
        aggregate_other_per_sec=None,  # type: float
        aggregate_share_metadata_reads_per_sec=None,  # type: float
        aggregate_usec_per_file_metadata_create_op=None,  # type: float
        aggregate_usec_per_file_metadata_modify_op=None,  # type: float
        aggregate_usec_per_file_metadata_read_op=None,  # type: float
        aggregate_usec_per_other_op=None,  # type: float
        aggregate_usec_per_share_metadata_read_op=None,  # type: float
        creates_per_sec=None,  # type: float
        fsinfos_per_sec=None,  # type: float
        fsstats_per_sec=None,  # type: float
        getattrs_per_sec=None,  # type: float
        links_per_sec=None,  # type: float
        lookups_per_sec=None,  # type: float
        mkdirs_per_sec=None,  # type: float
        pathconfs_per_sec=None,  # type: float
        reads_per_sec=None,  # type: float
        readdirs_per_sec=None,  # type: float
        readdirpluses_per_sec=None,  # type: float
        readlinks_per_sec=None,  # type: float
        removes_per_sec=None,  # type: float
        renames_per_sec=None,  # type: float
        rmdirs_per_sec=None,  # type: float
        setattrs_per_sec=None,  # type: float
        symlinks_per_sec=None,  # type: float
        time=None,  # type: int
        writes_per_sec=None,  # type: float
        usec_per_access_op=None,  # type: float
        usec_per_create_op=None,  # type: float
        usec_per_fsinfo_op=None,  # type: float
        usec_per_fsstat_op=None,  # type: float
        usec_per_getattr_op=None,  # type: float
        usec_per_link_op=None,  # type: float
        usec_per_lookup_op=None,  # type: float
        usec_per_mkdir_op=None,  # type: float
        usec_per_pathconf_op=None,  # type: float
        usec_per_read_op=None,  # type: float
        usec_per_readdir_op=None,  # type: float
        usec_per_readdirplus_op=None,  # type: float
        usec_per_readlink_op=None,  # type: float
        usec_per_remove_op=None,  # type: float
        usec_per_rename_op=None,  # type: float
        usec_per_rmdir_op=None,  # type: float
        usec_per_setattr_op=None,  # type: float
        usec_per_symlink_op=None,  # type: float
        usec_per_write_op=None,  # type: float
    ):
        """
        Keyword args:
            name (str): Name of the object (e.g., a file system or snapshot).
            id (str): A non-modifiable, globally unique ID chosen by the system.
            accesses_per_sec (float): ACCESS requests processed per second.
            aggregate_file_metadata_creates_per_sec (float): Sum of file-level or directory-level create-like metadata requests per second. Includes CREATE, LINK, MKDIR, and SYMLINK.
            aggregate_file_metadata_modifies_per_sec (float): Sum of file-level or directory-level modify-like and delete-like metadata requests per second. Includes REMOVE, RENAME, RMDIR, and SETATTR.
            aggregate_file_metadata_reads_per_sec (float): Sum of file-level or directory-level read-like metadata requests per second. Includes GETATTR, LOOKUP, PATHCONF, READDIR, READDIRPLUS, and READLINK.
            aggregate_other_per_sec (float): Sum of all other requests processed per second. This includes COMMIT, MKNOD, NULL, NLM operations, and operations which are supported for NFSv4.1 but not NFSv3.
            aggregate_share_metadata_reads_per_sec (float): Sum of share-level read-like metadata requests per second. Includes ACCESS, FSINFO, and FSSTAT.
            aggregate_usec_per_file_metadata_create_op (float): Average time, measured in microseconds, it takes the array to process a file-level or directory-level create-like metadata request. Includes CREATE, LINK, MKDIR, and SYMLINK.
            aggregate_usec_per_file_metadata_modify_op (float): Average time, measured in microseconds, it takes the array to process a file-level or directory-level modify-like or delete-like metadata request. Includes REMOVE, RENAME, RMDIR, and SETATTR.
            aggregate_usec_per_file_metadata_read_op (float): Average time, measured in microseconds, it takes the array to process a file-level or directory-level read-like metadata request. Includes GETATTR, LOOKUP, PATHCONF, READDIR, READDIRPLUS, and READLINK.
            aggregate_usec_per_other_op (float): Average time, measured in microseconds, it takes the array to process all other requests. This includes COMMIT, MKNOD, NULL, NLM operations, and operations which are supported for NFSv4.1 but not NFSv3.
            aggregate_usec_per_share_metadata_read_op (float): Average time, measured in microseconds, it takes the array to process a share-level read-like metadata request. Includes ACCESS, FSINFO, and FSSTAT.
            creates_per_sec (float): CREATE requests processed per second.
            fsinfos_per_sec (float): FSINFO requests processed per second.
            fsstats_per_sec (float): FSSTAT requests processed per second.
            getattrs_per_sec (float): GETATTR requests processed per second.
            links_per_sec (float): LINK requests processed per second.
            lookups_per_sec (float): LOOKUP requests processed per second.
            mkdirs_per_sec (float): MKDIR requests processed per second.
            pathconfs_per_sec (float): PATHCONF requests processed per second.
            reads_per_sec (float): READ requests processed per second.
            readdirs_per_sec (float): READDIR requests processed per second.
            readdirpluses_per_sec (float): READDIRPLUS requests processed per second.
            readlinks_per_sec (float): READLINK requests processed per second.
            removes_per_sec (float): REMOVE requests processed per second.
            renames_per_sec (float): RENAME requests processed per second.
            rmdirs_per_sec (float): RMDIR requests processed per second.
            setattrs_per_sec (float): SETATTR requests processed per second.
            symlinks_per_sec (float): SYMLINK requests processed per second.
            time (int): Sample time in milliseconds since UNIX epoch.
            writes_per_sec (float): WRITE requests processed per second.
            usec_per_access_op (float): Average time, measured in microseconds, it takes the array to process an ACCESS request.
            usec_per_create_op (float): Average time, measured in microseconds, it takes the array to process a CREATE request.
            usec_per_fsinfo_op (float): Average time, measured in microseconds, it takes the array to process an FSINFO request.
            usec_per_fsstat_op (float): Average time, measured in microseconds, it takes the array to process an FSSTAT request.
            usec_per_getattr_op (float): Average time, measured in microseconds, it takes the array to process a GETATTR request.
            usec_per_link_op (float): Average time, measured in microseconds, it takes the array to process a LINK request.
            usec_per_lookup_op (float): Average time, measured in microseconds, it takes the array to process a LOOKUP request.
            usec_per_mkdir_op (float): Average time, measured in microseconds, it takes the array to process a MKDIR request.
            usec_per_pathconf_op (float): Average time, measured in microseconds, it takes the array to process a PATHCONF request.
            usec_per_read_op (float): Average time, measured in microseconds, it takes the array to process a READ request.
            usec_per_readdir_op (float): Average time, measured in microseconds, it takes the array to process a READDIR request.
            usec_per_readdirplus_op (float): Average time, measured in microseconds, it takes the array to process a READDIRPLUS request.
            usec_per_readlink_op (float): Average time, measured in microseconds, it takes the array to process a READLINK request.
            usec_per_remove_op (float): Average time, measured in microseconds, it takes the array to process a REMOVE request.
            usec_per_rename_op (float): Average time, measured in microseconds, it takes the array to process a RENAME request.
            usec_per_rmdir_op (float): Average time, measured in microseconds, it takes the array to process an RMDIR request.
            usec_per_setattr_op (float): Average time, measured in microseconds, it takes the array to process a SETATTR request.
            usec_per_symlink_op (float): Average time, measured in microseconds, it takes the array to process a SYMLINK request.
            usec_per_write_op (float): Average time, measured in microseconds, it takes the array to process a WRITE request.
        """
        if name is not None:
            self.name = name
        if id is not None:
            self.id = id
        if accesses_per_sec is not None:
            self.accesses_per_sec = accesses_per_sec
        if aggregate_file_metadata_creates_per_sec is not None:
            self.aggregate_file_metadata_creates_per_sec = aggregate_file_metadata_creates_per_sec
        if aggregate_file_metadata_modifies_per_sec is not None:
            self.aggregate_file_metadata_modifies_per_sec = aggregate_file_metadata_modifies_per_sec
        if aggregate_file_metadata_reads_per_sec is not None:
            self.aggregate_file_metadata_reads_per_sec = aggregate_file_metadata_reads_per_sec
        if aggregate_other_per_sec is not None:
            self.aggregate_other_per_sec = aggregate_other_per_sec
        if aggregate_share_metadata_reads_per_sec is not None:
            self.aggregate_share_metadata_reads_per_sec = aggregate_share_metadata_reads_per_sec
        if aggregate_usec_per_file_metadata_create_op is not None:
            self.aggregate_usec_per_file_metadata_create_op = aggregate_usec_per_file_metadata_create_op
        if aggregate_usec_per_file_metadata_modify_op is not None:
            self.aggregate_usec_per_file_metadata_modify_op = aggregate_usec_per_file_metadata_modify_op
        if aggregate_usec_per_file_metadata_read_op is not None:
            self.aggregate_usec_per_file_metadata_read_op = aggregate_usec_per_file_metadata_read_op
        if aggregate_usec_per_other_op is not None:
            self.aggregate_usec_per_other_op = aggregate_usec_per_other_op
        if aggregate_usec_per_share_metadata_read_op is not None:
            self.aggregate_usec_per_share_metadata_read_op = aggregate_usec_per_share_metadata_read_op
        if creates_per_sec is not None:
            self.creates_per_sec = creates_per_sec
        if fsinfos_per_sec is not None:
            self.fsinfos_per_sec = fsinfos_per_sec
        if fsstats_per_sec is not None:
            self.fsstats_per_sec = fsstats_per_sec
        if getattrs_per_sec is not None:
            self.getattrs_per_sec = getattrs_per_sec
        if links_per_sec is not None:
            self.links_per_sec = links_per_sec
        if lookups_per_sec is not None:
            self.lookups_per_sec = lookups_per_sec
        if mkdirs_per_sec is not None:
            self.mkdirs_per_sec = mkdirs_per_sec
        if pathconfs_per_sec is not None:
            self.pathconfs_per_sec = pathconfs_per_sec
        if reads_per_sec is not None:
            self.reads_per_sec = reads_per_sec
        if readdirs_per_sec is not None:
            self.readdirs_per_sec = readdirs_per_sec
        if readdirpluses_per_sec is not None:
            self.readdirpluses_per_sec = readdirpluses_per_sec
        if readlinks_per_sec is not None:
            self.readlinks_per_sec = readlinks_per_sec
        if removes_per_sec is not None:
            self.removes_per_sec = removes_per_sec
        if renames_per_sec is not None:
            self.renames_per_sec = renames_per_sec
        if rmdirs_per_sec is not None:
            self.rmdirs_per_sec = rmdirs_per_sec
        if setattrs_per_sec is not None:
            self.setattrs_per_sec = setattrs_per_sec
        if symlinks_per_sec is not None:
            self.symlinks_per_sec = symlinks_per_sec
        if time is not None:
            self.time = time
        if writes_per_sec is not None:
            self.writes_per_sec = writes_per_sec
        if usec_per_access_op is not None:
            self.usec_per_access_op = usec_per_access_op
        if usec_per_create_op is not None:
            self.usec_per_create_op = usec_per_create_op
        if usec_per_fsinfo_op is not None:
            self.usec_per_fsinfo_op = usec_per_fsinfo_op
        if usec_per_fsstat_op is not None:
            self.usec_per_fsstat_op = usec_per_fsstat_op
        if usec_per_getattr_op is not None:
            self.usec_per_getattr_op = usec_per_getattr_op
        if usec_per_link_op is not None:
            self.usec_per_link_op = usec_per_link_op
        if usec_per_lookup_op is not None:
            self.usec_per_lookup_op = usec_per_lookup_op
        if usec_per_mkdir_op is not None:
            self.usec_per_mkdir_op = usec_per_mkdir_op
        if usec_per_pathconf_op is not None:
            self.usec_per_pathconf_op = usec_per_pathconf_op
        if usec_per_read_op is not None:
            self.usec_per_read_op = usec_per_read_op
        if usec_per_readdir_op is not None:
            self.usec_per_readdir_op = usec_per_readdir_op
        if usec_per_readdirplus_op is not None:
            self.usec_per_readdirplus_op = usec_per_readdirplus_op
        if usec_per_readlink_op is not None:
            self.usec_per_readlink_op = usec_per_readlink_op
        if usec_per_remove_op is not None:
            self.usec_per_remove_op = usec_per_remove_op
        if usec_per_rename_op is not None:
            self.usec_per_rename_op = usec_per_rename_op
        if usec_per_rmdir_op is not None:
            self.usec_per_rmdir_op = usec_per_rmdir_op
        if usec_per_setattr_op is not None:
            self.usec_per_setattr_op = usec_per_setattr_op
        if usec_per_symlink_op is not None:
            self.usec_per_symlink_op = usec_per_symlink_op
        if usec_per_write_op is not None:
            self.usec_per_write_op = usec_per_write_op

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `ArrayNfsSpecificPerformance`".format(key))
        if key == "accesses_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `accesses_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_file_metadata_creates_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_file_metadata_creates_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_file_metadata_modifies_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_file_metadata_modifies_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_file_metadata_reads_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_file_metadata_reads_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_other_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_other_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_share_metadata_reads_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_share_metadata_reads_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_usec_per_file_metadata_create_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_usec_per_file_metadata_create_op`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_usec_per_file_metadata_modify_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_usec_per_file_metadata_modify_op`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_usec_per_file_metadata_read_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_usec_per_file_metadata_read_op`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_usec_per_other_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_usec_per_other_op`, must be a value greater than or equal to `0.0`")
        if key == "aggregate_usec_per_share_metadata_read_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `aggregate_usec_per_share_metadata_read_op`, must be a value greater than or equal to `0.0`")
        if key == "creates_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `creates_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "fsinfos_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `fsinfos_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "fsstats_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `fsstats_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "getattrs_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `getattrs_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "links_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `links_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "lookups_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `lookups_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "mkdirs_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `mkdirs_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "pathconfs_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `pathconfs_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "reads_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `reads_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "readdirs_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `readdirs_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "readdirpluses_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `readdirpluses_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "readlinks_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `readlinks_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "removes_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `removes_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "renames_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `renames_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "rmdirs_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `rmdirs_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "setattrs_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `setattrs_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "symlinks_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `symlinks_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "writes_per_sec" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `writes_per_sec`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_access_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_access_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_create_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_create_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_fsinfo_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_fsinfo_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_fsstat_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_fsstat_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_getattr_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_getattr_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_link_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_link_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_lookup_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_lookup_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_mkdir_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_mkdir_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_pathconf_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_pathconf_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_read_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_read_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_readdir_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_readdir_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_readdirplus_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_readdirplus_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_readlink_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_readlink_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_remove_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_remove_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_rename_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_rename_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_rmdir_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_rmdir_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_setattr_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_setattr_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_symlink_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_symlink_op`, must be a value greater than or equal to `0.0`")
        if key == "usec_per_write_op" and value is not None:
            if value < 0.0:
                raise ValueError("Invalid value for `usec_per_write_op`, must be a value greater than or equal to `0.0`")
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            return None
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(ArrayNfsSpecificPerformance, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ArrayNfsSpecificPerformance):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashBlade REST API

    A lightweight client for FlashBlade REST API 2.3, developed by Pure Storage, Inc. (http://www.purestorage.com/).

    OpenAPI spec version: 2.3
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flashblade.FB_2_3 import models

class ActiveDirectoryPatch(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'directory_servers': 'list[str]',
        'encryption_types': 'list[str]',
        'fqdns': 'list[str]',
        'join_ou': 'str',
        'kerberos_servers': 'list[str]',
        'service_principal_names': 'list[str]'
    }

    attribute_map = {
        'directory_servers': 'directory_servers',
        'encryption_types': 'encryption_types',
        'fqdns': 'fqdns',
        'join_ou': 'join_ou',
        'kerberos_servers': 'kerberos_servers',
        'service_principal_names': 'service_principal_names'
    }

    required_args = {
    }

    def __init__(
        self,
        directory_servers=None,  # type: List[str]
        encryption_types=None,  # type: List[str]
        fqdns=None,  # type: List[str]
        join_ou=None,  # type: str
        kerberos_servers=None,  # type: List[str]
        service_principal_names=None,  # type: List[str]
    ):
        """
        Keyword args:
            directory_servers (list[str]): A list of directory servers that will be used for lookups related to user authorization. Accepted server formats are IP address and DNS name. All specified servers must be registered to the domain appropriately in the array's configured DNS and will only be communicated with over the secure LDAP (LDAPS) protocol. If not specified, servers are resolved for the domain in DNS. The specified list can have a maximum length of 5.
            encryption_types (list[str]): The encryption types that will be supported for use by clients for Kerberos authentication. Valid values include `aes256-cts-hmac-sha1-96`, `aes128-cts-hmac-sha1-96`, and `arcfour-hmac`.
            fqdns (list[str]): A list of fully qualified domain names to use to register service principal names for the machine account. If specified, every service principal that is supported by the array will be registered for each fully qualified domain name specified. If neither `fqdns` nor `service_principal_names` is specified, the default `service_principal_names` are constructed using the `computer_name` and `domain` fields. Cannot be provided in combination with `service_principal_names`.
            join_ou (str): The relative distinguished name of the organizational unit in which the computer account should be created when joining the domain.
            kerberos_servers (list[str]): A list of key distribution servers to use for Kerberos protocol. Accepted server formats are IP address and DNS name. All specified servers must be registered to the domain appropriately in the array's configured DNS. If not specified, servers are resolved for the domain in DNS. The specified list can have a maximum length of 5.
            service_principal_names (list[str]): A list of service principal names to register for the machine account, which can be used for the creation of keys for Kerberos authentication. If neither `service_principal_names` nor `fqdns` is specified, the default `service_principal_names` are constructed using the `computer_name` and `domain` fields. Cannot be provided in combination with `fqdns`.
        """
        if directory_servers is not None:
            self.directory_servers = directory_servers
        if encryption_types is not None:
            self.encryption_types = encryption_types
        if fqdns is not None:
            self.fqdns = fqdns
        if join_ou is not None:
            self.join_ou = join_ou
        if kerberos_servers is not None:
            self.kerberos_servers = kerberos_servers
        if service_principal_names is not None:
            self.service_principal_names = service_principal_names

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `ActiveDirectoryPatch`".format(key))
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            return None
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(ActiveDirectoryPatch, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ActiveDirectoryPatch):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    FlashArray REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 2.6
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re

import six
import typing

from ....properties import Property
if typing.TYPE_CHECKING:
    from pypureclient.flasharray.FA_2_6 import models

class NetworkInterfacePatch(object):
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'enabled': 'bool',
        'eth': 'NetworkinterfacepatchEth',
        'override_npiv_check': 'bool'
    }

    attribute_map = {
        'enabled': 'enabled',
        'eth': 'eth',
        'override_npiv_check': 'override_npiv_check'
    }

    required_args = {
    }

    def __init__(
        self,
        enabled=None,  # type: bool
        eth=None,  # type: models.NetworkinterfacepatchEth
        override_npiv_check=None,  # type: bool
    ):
        """
        Keyword args:
            enabled (bool): Returns a value of `true` if the specified network interface or Fibre Channel port is enabled. Returns a value of `false` if the specified network interface or Fibre Channel port is disabled.
            eth (NetworkinterfacepatchEth)
            override_npiv_check (bool): N-Port ID Virtualization (NPIV) requires a balanced configuration of Fibre Channel ports configured for SCSI on both controllers. Enabling or Disabling a Fibre Channel port configured for SCSI might cause the NPIV status to change from enabled to disabled or vice versa. Set this option to proceed with enabling or disabling the port.
        """
        if enabled is not None:
            self.enabled = enabled
        if eth is not None:
            self.eth = eth
        if override_npiv_check is not None:
            self.override_npiv_check = override_npiv_check

    def __setattr__(self, key, value):
        if key not in self.attribute_map:
            raise KeyError("Invalid key `{}` for `NetworkInterfacePatch`".format(key))
        self.__dict__[key] = value

    def __getattribute__(self, item):
        value = object.__getattribute__(self, item)
        if isinstance(value, Property):
            raise AttributeError
        else:
            return value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            if hasattr(self, attr):
                value = getattr(self, attr)
                if isinstance(value, list):
                    result[attr] = list(map(
                        lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                        value
                    ))
                elif hasattr(value, "to_dict"):
                    result[attr] = value.to_dict()
                elif isinstance(value, dict):
                    result[attr] = dict(map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict") else item,
                        value.items()
                    ))
                else:
                    result[attr] = value
        if issubclass(NetworkInterfacePatch, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NetworkInterfacePatch):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
